function [ OUTPUT ] = ColdCavityApproachFunction( INPUT, C , DoPlots)
CC.I = INPUT;

% INPUT - input paramteres structure
% C - physical constants
% CC.I.Fspan can be defined in two ways:
% (95.76:0.001:95.78)  - vector of values 
% (95, 0.001)          - vector of two numbers, first is frequency, second
% is search step

if ~isfield(CC.I, 'Fspan')
    CC.I.Fspan=0;
    disp('Frequency or frequency span not input. I am generating frequency search range automatically.')
end


%% Initial calculations
CC.ResonatorAngleTanges = tan(CC.I.ResonatorAngle*pi/180);
% For TE mode we need zero of first derivative
CC.CharacteristicValue = bessel_deriv_zeros(CC.I.ModM,CC.I.ModK);

s=sprintf('Characteristic value = %.11f', CC.CharacteristicValue);disp(s);

CC.ResonatorTotalLength = sum(CC.I.ResonatorLength);
CC.ResonatorCentralLength = CC.I.ResonatorLength(2);
CC.ResonatorSliceCount = CC.ResonatorTotalLength / CC.I.StepZ;
CC.AxisZ = CC.I.StepZ*(0:CC.ResonatorSliceCount)';

%% Cavity dimmension calculation - radius vector
R1 = CC.I.ResonatorRadiusR0 + (CC.AxisZ-CC.I.ResonatorLength(1))*CC.ResonatorAngleTanges(1);
R2 = CC.I.ResonatorRadiusR0 + (CC.AxisZ-CC.I.ResonatorLength(1))*CC.ResonatorAngleTanges(2);
R3 = CC.I.ResonatorRadiusR0 + CC.I.ResonatorLength(2)*CC.ResonatorAngleTanges(2) + (CC.AxisZ-(CC.I.ResonatorLength(1)+CC.I.ResonatorLength(2)))*CC.ResonatorAngleTanges(3);
n1 = fix(CC.I.ResonatorLength(1)/CC.I.StepZ)+1;
n2 = fix((CC.I.ResonatorLength(1)+CC.I.ResonatorLength(2))/CC.I.StepZ)+1;
CC.ResonatorRadius = [R1(1:n1);R2(n1+1:n2);R3(n2+1:end)]; % R(z)

if DoPlots == 1
    plot(CC.AxisZ,R1,'b:',CC.AxisZ,R2,'b--',CC.AxisZ,R3,'b-.',CC.AxisZ,CC.ResonatorRadius,'r'); 
    set(gcf(),'Name','R(z)'); xlabel('z [mm]'); ylabel('R(z) [mm]');
end

clearvars -except C CC DoPlots  % Remove temporary variables except needed

%% Optimize f and Q to fullfil boundary conditions - minimalization
%   RFfield equation is solved for different f (frequency) Q (diffractive
%   quality and we search for minimal of equation |R| (boundary condition)

if (size(CC.I.Fspan,2) > 2)  % If frequency search span was input we use it
    Fspan = CC.I.Fspan;
    lambda = C.SpeedOfLight/mean(CC.I.Fspan); 
else % If one frequency value was input we generate search region for freq.
    
    if (size(CC.I.Fspan,2) == 1), Fstep = 0.001;
    else Fstep = CC.I.Fspan(2); % If 2 elements present
    end
   
    f_min = (C.SpeedOfLight/2/pi)*sqrt((CC.CharacteristicValue/CC.I.ResonatorRadiusR0)^2+(pi/CC.ResonatorTotalLength)^2);
    f_max = (C.SpeedOfLight/2/pi)*sqrt((CC.CharacteristicValue/CC.I.ResonatorRadiusR0)^2+(pi/CC.ResonatorCentralLength)^2);
    Fspan = (f_min:Fstep:f_max);
    lambda = C.SpeedOfLight/mean(Fspan);
end


if isfield(CC.I, 'Qspan') &&  (size(CC.I.Fspan,2) > 1)   
        % If resonator quality Q search range was input we use it
        Qspan = CC.I.Qspan;      
else
    
    if isfield(CC.I, 'Qspan')
        % If only Q accuracy step was input we use it
        Qstep = CC.I.Qspan;
    else
        Qstep = 0.1; % No accurace or range was input we generate Q span
    end
    
    % Oblicz Qspan
    join_len = 0; % paraboliczne laczenia
    len = CC.I.ResonatorLength'; % tymczasowo tak, az dorobie liczenie innych rezonatorow

    Qdiff_min = 4*pi*(((CC.ResonatorCentralLength+2*join_len)/lambda)^2);

    n_centr = 1+(size(len,2)-1)/2;
    %Qdiff_max=4*pi*(((len(n_centr)+len(n_centr-1)/3+len(n_centr+1)/3)/lambda)^2);
    Qdiff_max=4*pi*(((CC.ResonatorTotalLength+2*join_len)/lambda)^2)*4; % idea for equation
    Qspan = (Qdiff_min:Qstep:Qdiff_max);
    
    
end


str = sprintf('Freq range f= %.4f - %.4f, Q range = %.4f - %.4f', min(Fspan), max(Fspan), min(Qspan), max(Qspan));disp(str);

% Optymisation
[ NewAxisZ, RFfield, Foptimal, Qoptimal, BoundaryConditionErrorOptimal ] = EqColdCavityFieldOptimization( C.SpeedOfLight, CC.CharacteristicValue, CC.ResonatorRadius, CC.AxisZ, Fspan, Qspan );

% Full review will check all posible combinations of f and Q and will
% create a plot absR.bmp - for full review decreas Qspan accurace to safe
% time
%[ NewAxisZ, RFfield, Foptimal, Qoptimal, BoundaryConditionErrorOptimal ] = EqColdCavityFullReview( C.SpeedOfLight, CC.CharacteristicValue, CC.ResonatorRadius, CC.AxisZ, Fspan, Qspan );


%% Resonator Quality calculations Qdiff Qtotal Qohmic
% ohmic quality
SkinDepth = sqrt(C.CopperConductivity/(pi*Foptimal*C.PermeabilityMi));

Qohm = CC.I.ResonatorRadiusR0*(1-CC.I.ModM^2/CC.CharacteristicValue^2)/SkinDepth; % Nusin (3.44)  Kartikeyan (5.11)

% total quality
Qdiff = Qoptimal;  
Qtot = 1/(1/Qohm+1/Qdiff);

%% Save results
CC.O.RFfieldRaw = RFfield;
CC.O.RFfieldNormalized = CC.O.RFfieldRaw/max(abs(CC.O.RFfieldRaw));
CC.O.RFfieldAxisZ = NewAxisZ;
CC.O.Foptimal = Foptimal;
CC.O.Qoptimal = Qoptimal;   % optimal Qdiff
CC.O.Qohm = Qohm;  % Q ohmic
CC.O.Qtotal = Qtot;  % Q total
CC.O.Qmin = Qspan(1);
CC.O.Qmax = Qspan(end);
CC.O.Fmin = Fspan(1);
CC.O.Fmax = Fspan(end);
CC.O.ResonatorTotalLength = CC.ResonatorTotalLength;
CC.O.ResonatorCentralLength = CC.ResonatorCentralLength;
CC.O.BoundaryConditionErrorOptimal = BoundaryConditionErrorOptimal;

% We store parameters of the resonator
CC.O.ResonatorLength = CC.I.ResonatorLength; 
CC.O.ResonatorAngle = CC.I.ResonatorAngle;
CC.O.ResonatorRadius = CC.ResonatorRadius;
CC.O.ResonatorRadiusR0 = CC.I.ResonatorRadiusR0;
CC.O.CharacteristicValue = CC.CharacteristicValue;
CC.O.StepZ = CC.I.StepZ;
CC.O.ModM = CC.I.ModM;
CC.O.ModK = CC.I.ModK;
CC.O.HarmonicNumber = CC.I.HarmonicNumber;
%ColdCavity.Input= CC.I;
%ColdCavity.Output = CC.O;
CC.O.Comment = 'Cold Cavity Output';

if DoPlots == 1
    %% Plot charts
    % for optimal f i Q

    figure();plot(NewAxisZ, real(RFfield), NewAxisZ, imag(RFfield), NewAxisZ,abs(RFfield));
    set(gcf(),'Name','fi(z)');xlabel('z [mm]'); ylabel('f(z) [Optimal]');

    figure();plot(NewAxisZ, abs(CC.O.RFfieldNormalized)*CC.I.ResonatorRadiusR0,CC.AxisZ,CC.ResonatorRadius);
    set(gcf(),'Name','RFfield(z)'); xlabel('z [mm]'); ylabel('RFfieldOptimal(z) [a.u.], R(z) [mm]');
    legend('RFfieldOptimal(z)','R(z)');
end
% Frequency and Q optimal
str = sprintf('Optimal Freq= %.4f, Optimal Q=%.4f', CC.O.Foptimal, CC.O.Qoptimal);disp(str);

%% Return result from function
OUTPUT = CC.O;


end

